@@ -12,20 +12,27 @@
 #-*
 #Z* -------------------------------------------------------------------
 
+from __future__ import print_function, absolute_import
+
 if __name__=='pymol.commanding':
 
-    import thread
-    import string
+    import sys
+    if sys.version_info[0] == 2:
+        import thread
+    else:
+        import _thread as thread
+        from io import FileIO as file
+
     import re
     import os
     import time
     import threading
     import traceback
-    import parsing
-    import cmd
+    from . import parsing
+    cmd = sys.modules["pymol.cmd"]
     import pymol
     
-    from cmd import _cmd, Shortcut, QuietException, \
+    from .cmd import _cmd, Shortcut, QuietException, \
           fb_module, fb_mask, is_list, \
           DEFAULT_ERROR, DEFAULT_SUCCESS, is_ok, is_error, is_string
 
@@ -83,7 +90,7 @@ SEE ALSO
             return s.rstrip(',') + ', async=0'
         def write(self, s):
             s = re_fetch.sub(self._append_async0, s)
-            file.write(self, s)
+            file.write(self, s.encode())
  
     def log_open(filename='log.pml', mode='w', _self=cmd):
         '''
@@ -116,9 +123,9 @@ SEE ALSO
                 pymol._log_file = LogFile(filename,mode)
                 if _self._feedback(fb_module.cmd,fb_mask.details): # redundant
                     if mode!='a':
-                        print " Cmd: logging to '%s'."%filename
+                        print(" Cmd: logging to '%s'."%filename)
                     else:
-                        print " Cmd: appending to '%s'."%filename            
+                        print(" Cmd: appending to '%s'."%filename)
                 if mode=='a':
                     pymol._log_file.write("\n") # always start on a new line
                 if(re.search(r"\.py$|\.PY$|\.pym$|\.PYM$",filename)):
@@ -126,7 +133,7 @@ SEE ALSO
                 else:
                     _self.set("logging",1,quiet=1)
             except:
-                print"Error: unable to open log file '%s'"%filename
+                print("Error: unable to open log file '%s'"%filename)
                 pymol._log_file = None
                 _self.set("logging",0,quiet=1)
                 traceback.print_exc()
@@ -138,26 +145,45 @@ DESCRIPTION
 
     "log" writes a command to the log file (if one is open).
 
+    `text` and/or `alt_text` must include the terminating line feed.
+
+ARGUMENTS
+
+    text = str: PyMOL command (optional if alt_text is given)
+    alt_text = str: Python expression (optional)
+
 SEE ALSO
 
     log_open, log_close
     
         '''
-        pymol=_self._pymol        
-        cmd=_self
-        if hasattr(pymol,"_log_file"):
-            if pymol._log_file!=None:
-                mode = _self.get_setting_int("logging")
-                if mode:
-                    if mode==1:
-                        if not text and alt_text:
-                            text = '/' + alt_text
-                        pymol._log_file.write(text)
-                    elif mode==2:
-                        if not alt_text:
-                            alt_text = "cmd.do('''%s''')\n" % text.strip()
-                        pymol._log_file.write(alt_text)
-                    pymol._log_file.flush()
+        # See also equivalent C impelemtation: PLog
+
+        log_file = getattr(_self._pymol, "_log_file", None)
+
+        if log_file is None:
+            return
+
+        mode = _self.get_setting_int("logging")
+
+        if mode == 1:
+            # .pml
+            if not text and alt_text:
+                text = '/' + alt_text
+        elif mode == 2:
+            # .py
+            if alt_text:
+                text = alt_text
+            elif text.startswith('/'):
+                text = text[1:]
+            else:
+                text = "cmd.do(" + repr(text.strip()) + ")\n"
+        else:
+            return
+
+        if text:
+            log_file.write(text)
+            log_file.flush()
 
     def log_close(_self=cmd):
         '''
@@ -182,7 +208,7 @@ SEE ALSO
                 del pymol._log_file
                 _self.set("logging",0,quiet=1)
                 if _self._feedback(fb_module.cmd,fb_mask.details): # redundant
-                    print " Cmd: log closed."
+                    print(" Cmd: log closed.")
 
     def cls(_self=cmd): 
         '''
@@ -242,7 +268,7 @@ USAGE
         else:
             if _self.get_setting_int("internal_feedback") > 0:
                 _self.set("text","1",quiet=1)
-            print
+            print()
             try:
                 _self.lock(_self)
                 r = _cmd.splash(_self._COb,0)
@@ -357,7 +383,7 @@ USAGE (PYTHON)
     from pymol import cmd
     cmd.do("load file.pdb")
         '''
-        r = DEFAULT_ERROR
+        r = DEFAULT_SUCCESS
         log = int(log)
         if is_list(commands):
             cmmd_list = commands
@@ -368,7 +394,7 @@ USAGE (PYTHON)
             defer = _self.get_setting_int("defer_updates")
             _self.set('defer_updates',1)
         for cmmd in cmmd_list:
-            lst = string.split(string.replace(cmmd,chr(13),chr(10)),chr(10))
+            lst = cmmd.splitlines()
             if len(lst)<2:
                 for a in lst:
                     if(len(a)):
@@ -377,8 +403,6 @@ USAGE (PYTHON)
                             r = _cmd.do(_self._COb,a,log,echo)
                         finally:
                             _self.unlock(r,_self)
-                    else:
-                        r = DEFAULT_SUCCESS
             else:
                 try:
                     _self.lock(_self)
@@ -390,8 +414,6 @@ USAGE (PYTHON)
                             if do_flush:
                                 _self.unlock(r,_self) # flushes
                                 _self.lock(_self)
-                        else:
-                            r = DEFAULT_SUCCESS
                 finally:
                     _self.unlock(r,_self)
         if n_cmmd>1:
@@ -590,7 +612,11 @@ DESCRIPTION
         _self = kwargs.pop('_self', cmd)
 
         wiz = Message(['please wait ...'], dismiss=0, _self=_self)
-        _self.set_wizard(wiz)
+
+        try:
+            _self.set_wizard(wiz)
+        except:
+            wiz = None
 
         if isinstance(func, str):
             func = _self.keyword[func][0]
@@ -601,11 +627,17 @@ DESCRIPTION
                 func(*args, **kwargs)
             except (pymol.CmdException, cmd.QuietException) as e:
                 if e.message:
-                    print e
+                    print(e)
             finally:
-                _self.set_wizard_stack(filter(lambda w: w != wiz,
-                    _self.get_wizard_stack()))
-                _self.refresh_wizard()
+                if wiz is not None:
+                    try:
+                        _self.set_wizard_stack([w
+                            for w in _self.get_wizard_stack() if w != wiz])
+                    except:
+                        _self.do('_ wizard')
+                    else:
+                        _self.refresh_wizard()
+
                 async_threads.remove(t)
 
         t = threading.Thread(target=wrapper)
